

////////////////////////////////////////////////////////////////////////


#ifndef  __VERIF_COMMAND_LINE_H__
#define  __VERIF_COMMAND_LINE_H__


////////////////////////////////////////////////////////////////////////


#include "vx_util/concat_string.h"
#include "vx_util/string_array.h"


////////////////////////////////////////////////////////////////////////


typedef void (*CLSetFunction)(const StringArray &);   //  command-line set function

typedef void (*UsageFunction)();   //  usage function


////////////////////////////////////////////////////////////////////////


class CLOptionInfo {   //  command-line option info

   private:

      void init_from_scratch();

      void assign(const CLOptionInfo &);

   public:

      CLOptionInfo();
     ~CLOptionInfo();
      CLOptionInfo(const CLOptionInfo &);
      CLOptionInfo & operator=(const CLOptionInfo &);

      void clear();

      void dump(ostream &, int = 0) const;


      ConcatString option_text;

      int Nargs;

      CLSetFunction f;   //  not allocated

};


////////////////////////////////////////////////////////////////////////


class CLOptionInfoArray {

   private:

      void init_from_scratch();

      void assign(const CLOptionInfoArray &);

      void extend(int);


      int Nelements;

      int Nalloc;

      int AllocInc;

      CLOptionInfo * e;


   public:

      CLOptionInfoArray();
     ~CLOptionInfoArray();
      CLOptionInfoArray(const CLOptionInfoArray &);
      CLOptionInfoArray & operator=(const CLOptionInfoArray &);

      void clear();

      void dump(ostream &, int = 0) const;

      void set_alloc_inc(int = 0);   //  0 means default value (16)

      int n_elements() const;

      void add(const CLOptionInfo &);
      void add(const CLOptionInfoArray &);

      CLOptionInfo & operator[](int) const;

      int lookup(const char *) const;

};


////////////////////////////////////////////////////////////////////////


inline int CLOptionInfoArray::n_elements() const { return ( Nelements ); }


////////////////////////////////////////////////////////////////////////


class CommandLine {

   private:

      void init_from_scratch();

      void assign(const CommandLine &);


      void do_help();

      StringArray args;

      CLOptionInfoArray options;

      ConcatString ProgramName;

      UsageFunction Usage;

   public:

      CommandLine();
     ~CommandLine();
      CommandLine(const CommandLine &);
      CommandLine & operator=(const CommandLine &);

      void clear();

      void dump(ostream &, int depth = 0) const;

      void set(int argc, char ** argv);   //  includes argv[0]

      void set_usage(UsageFunction);

      int n() const;  //  # of elements

      int max_length() const;

      void shift_down(int pos, int k);

      int has_option(int & index) const;

      int next_option() const;   //  -1 if no option found, else index into array

      const char * operator[](int) const;


      void parse();

      void add(CLSetFunction, const char * text, int n_args);   //  n_args not including switch

};


////////////////////////////////////////////////////////////////////////


inline int CommandLine::n() const { return ( args.n_elements() ); }

inline int CommandLine::max_length() const { return ( args.max_length() ); }

inline void CommandLine::set_usage(UsageFunction f) { Usage = f;  return; }


////////////////////////////////////////////////////////////////////////


#endif   /*  __VERIF_COMMAND_LINE_H__  */


////////////////////////////////////////////////////////////////////////


